/*
 *  Copyright 2010 Alexey Frishman (alexey.frishman@gmail.com)
 *  
 *  Licensed under the Apache License, Version 2.0 (the "License"); you may not
 *  use this file except in compliance with the License. You may obtain a copy of
 *  the License at
 *  
 *  http://www.apache.org/licenses/LICENSE-2.0
 *  
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *  License for the specific language governing permissions and limitations under
 *  the License.
 * 
 */

package org.json4g.serialization;

import static org.json4g.TypeUtil.*;
import com.google.gwt.json.client.JSONValue;
import com.gwtent.reflection.client.Field;
import com.gwtent.reflection.client.Type;

/**
 * Base abstract implementation for most part of serializers, which defines common behavior and convenient utility methods
 */
public abstract class AbstractJsonSerializer implements JsonSerializer {
    private Type[] supportedTypes;

    public AbstractJsonSerializer(Type... supportedTypes) {
        this.supportedTypes = supportedTypes;
    }

    @Override
    public boolean supports(Object object) {
        return object != null && supports(getType(object.getClass()));
    }

    @Override
    public boolean supports(JSONValue json, Field field) {
        return supports(field.getType()) && supports(json);
    }

    protected boolean supports(Type type) {
        if (type == null) {
            return false;
        }
        boolean result = false;
        for (Type t : supportedTypes) {
            if (typesAreEqual(type, t)) {
                result = true;
                break;
            }
        }
        return result;
    }

    protected boolean supports(JSONValue json) {
        return json != null && json.isNull() == null;
    }

    protected void assertSupported(Object object) {
        if (!supports(object)) {
            throw new IllegalArgumentException("Object '" + object + "' is not supported");
        }
    }

    protected void assertSupported(JSONValue json, Type type) {
        assertSupported(json);
        assertSupported(type);
    }
    
    protected void assertSupported(JSONValue json) {
        if (!supports(json)) {
            throw new IllegalArgumentException("JSON value '" + json + "' is not supported");
        }
    }

    protected void assertSupported(Type type) {
        if (!supports(type)) {
            throw new IllegalArgumentException("Type '" + type + "' is not supported");
        }
    }
}
