/*
 *  Copyright 2010 Alexey Frishman (alexey.frishman@gmail.com)
 *  
 *  Licensed under the Apache License, Version 2.0 (the "License"); you may not
 *  use this file except in compliance with the License. You may obtain a copy of
 *  the License at
 *  
 *  http://www.apache.org/licenses/LICENSE-2.0
 *  
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *  License for the specific language governing permissions and limitations under
 *  the License.
 * 
 */

package org.json4g.serialization.parameterized;

import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.TreeSet;

import org.json4g.serialization.JsonSerializer;
import org.json4g.serialization.policy.SerializationPolicy;

import com.gwtent.reflection.client.impl.ClassTypeImpl;

public class JsonSetSerializer extends JsonCollectionSerializer {

    public JsonSetSerializer(JsonSerializer itemSerializer, SerializationPolicy policy) {
        super(itemSerializer, policy, 
                new ClassTypeImpl(Set.class),
                new ClassTypeImpl(HashSet.class),
                new ClassTypeImpl(LinkedHashSet.class),
                new ClassTypeImpl(TreeSet.class)
        );
    }

    @SuppressWarnings("unchecked")
    @Override
    protected Collection<?> createNewInstance(int size) {
        // TODO LinkedHashSet preserves order of elements, this may be helpful,
        // if originally (before serialization to JSON) it was TreeSet or LinkedHashSet.
        // In such case it will contain elements in the same order, as original set did.
        // Think how to customize this and allow to create appropriate implementation, the same, as on server
        return new LinkedHashSet(size); 
    }

}
