/*
 *  Copyright 2010 Alexey Frishman (alexey.frishman@gmail.com)
 *  
 *  Licensed under the Apache License, Version 2.0 (the "License"); you may not
 *  use this file except in compliance with the License. You may obtain a copy of
 *  the License at
 *  
 *  http://www.apache.org/licenses/LICENSE-2.0
 *  
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *  License for the specific language governing permissions and limitations under
 *  the License.
 * 
 */

package org.json4g.serialization.policy.jaxb;

import javax.xml.bind.annotation.XmlElement;

import org.json4g.serialization.policy.FieldSerializationCondition;

import com.gwtent.reflection.client.Field;

/**
 * Condition based on features of JAXB annotation @XmlElement
 */
public class JaxbSerializationCondition implements FieldSerializationCondition {

    /**
     * Allows to serialize only if @XmlElement is present.
     * Allows to serialize nulls only if @XmlElement(nillable=true)
     */
    @Override
    public boolean isSatisfiedForSerialization(Object value, Field field) {
        XmlElement annotation = field.getAnnotation(XmlElement.class);
        return annotation != null && (value != null || annotation.nillable());
    }

    /**
     * Allows to deserialize only if @XmlElement is present.
     */
    @Override
    public boolean isSatisfiedForDeserialization(Field field) {
        return field != null && field.isAnnotationPresent(XmlElement.class);
    }

}
