/*
 *  Copyright 2010 Alexey Frishman (alexey.frishman@gmail.com)
 *  
 *  Licensed under the Apache License, Version 2.0 (the "License"); you may not
 *  use this file except in compliance with the License. You may obtain a copy of
 *  the License at
 *  
 *  http://www.apache.org/licenses/LICENSE-2.0
 *  
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *  License for the specific language governing permissions and limitations under
 *  the License.
 * 
 */

package org.json4g.serialization.primitive;

import static org.json4g.TypeUtil.getType;

import org.json4g.serialization.AbstractJsonSerializer;

import com.google.gwt.json.client.JSONBoolean;
import com.google.gwt.json.client.JSONValue;
import com.gwtent.reflection.client.Field;
import com.gwtent.reflection.client.PrimitiveType;

public class JsonBooleanSerializer extends AbstractJsonSerializer {

    public JsonBooleanSerializer() {
        super(getType(Boolean.class), PrimitiveType.BOOLEAN);
    }

    @Override
    public Object deserialize(JSONValue json, Field field) {
        assertSupported(json, field.getType());
        return Boolean.valueOf(json.isBoolean().booleanValue());
    }

    @Override
    public JSONValue serialize(Object object) {
        assertSupported(object);
        return JSONBoolean.getInstance((Boolean) object);
    }
    
    @Override
    protected boolean supports(JSONValue json) {
        return super.supports(json) && json.isBoolean() != null;
    }
}
