/*
 *  Copyright 2010 Alexey Frishman (alexey.frishman@gmail.com)
 *  
 *  Licensed under the Apache License, Version 2.0 (the "License"); you may not
 *  use this file except in compliance with the License. You may obtain a copy of
 *  the License at
 *  
 *  http://www.apache.org/licenses/LICENSE-2.0
 *  
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *  License for the specific language governing permissions and limitations under
 *  the License.
 * 
 */

package org.json4g.serialization.primitive;

import org.json4g.serialization.AbstractJsonSerializer;

import com.google.gwt.json.client.JSONNumber;
import com.google.gwt.json.client.JSONValue;
import com.gwtent.reflection.client.Field;
import com.gwtent.reflection.client.Type;

public class JsonNumberSerializer extends AbstractJsonSerializer {

    public JsonNumberSerializer(Type... supportedTypes) {
        super(supportedTypes);
    }

    /**
     * Unfortunately must return Double values only. This is customized in other
     * serializers for Long, Integer, etc.
     */
    @Override
    public Object deserialize(JSONValue json, Field field) {
        return Double.valueOf(json.isNumber().doubleValue());
    }

    @Override
    public JSONValue serialize(Object object) {
        assertSupported(object);
        return new JSONNumber(((Number) object).doubleValue());
    }
    
    @Override
    protected boolean supports(JSONValue json) {
        return super.supports(json) && json.isNumber() != null;
    }

}
